/*
 * Copyright (C) 2021 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.android.eventlib.premade;

import android.app.Activity;
import android.os.Bundle;
import android.os.PersistableBundle;

import com.android.eventlib.events.activities.ActivityCreatedEvent;
import com.android.eventlib.events.activities.ActivityDestroyedEvent;
import com.android.eventlib.events.activities.ActivityPausedEvent;
import com.android.eventlib.events.activities.ActivityRestartedEvent;
import com.android.eventlib.events.activities.ActivityResumedEvent;
import com.android.eventlib.events.activities.ActivityStartedEvent;
import com.android.eventlib.events.activities.ActivityStoppedEvent;

/**
 * An {@link Activity} which logs events for all lifecycle events.
 */
public class EventLibActivity extends Activity {

    private String mOverrideActivityClassName;

    public void setOverrideActivityClassName(String overrideActivityClassName) {
        mOverrideActivityClassName = overrideActivityClassName;
    }

    /** Log a {@link ActivityCreatedEvent}. */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        logOnCreate(savedInstanceState, /* persistentState= */ null);
    }

    /** Log a {@link ActivityCreatedEvent}. */
    @Override
    public void onCreate(Bundle savedInstanceState, PersistableBundle persistentState) {
        super.onCreate(savedInstanceState, persistentState);
        logOnCreate(savedInstanceState, persistentState);
    }

    private void logOnCreate(Bundle savedInstanceState, PersistableBundle persistentState) {
        ActivityCreatedEvent.ActivityCreatedEventLogger logger =
                ActivityCreatedEvent.logger(this, savedInstanceState)
                        .setPersistentState(persistentState);

        if (mOverrideActivityClassName != null) {
            logger.setActivity(mOverrideActivityClassName);
        }

        logger.log();
    }

    /** Log a {@link ActivityStartedEvent}. */
    @Override
    protected void onStart() {
        super.onStart();
        ActivityStartedEvent.ActivityStartedEventLogger logger =
                ActivityStartedEvent.logger(this);

        if (mOverrideActivityClassName != null) {
            logger.setActivity(mOverrideActivityClassName);
        }

        logger.log();
    }

    /** Log a {@link ActivityRestartedEvent}. */
    @Override
    protected void onRestart() {
        super.onRestart();
        ActivityRestartedEvent.ActivityRestartedEventLogger logger =
                ActivityRestartedEvent.logger(this);

        if (mOverrideActivityClassName != null) {
            logger.setActivity(mOverrideActivityClassName);
        }

        logger.log();
    }

    /** Log a {@link ActivityResumedEvent}. */
    @Override
    protected void onResume() {
        super.onResume();
        ActivityResumedEvent.ActivityResumedEventLogger logger =
                ActivityResumedEvent.logger(this);

        if (mOverrideActivityClassName != null) {
            logger.setActivity(mOverrideActivityClassName);
        }

        logger.log();
    }

    /** Log a {@link ActivityPausedEvent}. */
    @Override
    protected void onPause() {
        super.onPause();
        ActivityPausedEvent.ActivityPausedEventLogger logger =
                ActivityPausedEvent.logger(this);

        if (mOverrideActivityClassName != null) {
            logger.setActivity(mOverrideActivityClassName);
        }

        logger.log();
    }

    /** Log a {@link ActivityStoppedEvent}. */
    @Override
    protected void onStop() {
        super.onStop();
        ActivityStoppedEvent.ActivityStoppedEventLogger logger =
                ActivityStoppedEvent.logger(this);

        if (mOverrideActivityClassName != null) {
            logger.setActivity(mOverrideActivityClassName);
        }

        logger.log();
    }

    /** Log a {@link ActivityDestroyedEvent}. */
    @Override
    protected void onDestroy() {
        super.onDestroy();
        ActivityDestroyedEvent.ActivityDestroyedEventLogger logger =
                ActivityDestroyedEvent.logger(this);

        if (mOverrideActivityClassName != null) {
            logger.setActivity(mOverrideActivityClassName);
        }

        logger.log();
    }
}
